"""
Pipeline to analyze previously known planets detected by TESS

argument 1: name of the planet
argument 2: kind of the analysis (global, occultation, etc.)
argument 3: type of the analysis (wouttess, withtess, onlytess)

Tansu Daylan
MIT Kavli Institute, Cambridge, MA, 02109, US
tansu.daylan@gmail.com
www.tansudaylan.com
"""

import numpy as np
import os, datetime, sys

import matplotlib.pyplot as plt

from allesfitter import config
import pickle

# base data path for the TESS known planet project
pathdata = os.environ['KNWN_DATA_PATH'] + '/'

os.system('mkdir -p %s' % pathdata)
os.system('mkdir -p %s' % (pathdata + '/postproc/'))

strgplan = sys.argv[1]
strgkind = sys.argv[2]

strgtimestmp = datetime.datetime.now().strftime('%Y%m%d_%H%M%S')
print('Post-processing comparison pipeline for the previously TESS exoplanets started at %s...' % strgtimestmp)
    

print('Will analyze:')
print(strgplan)

# threshold p value to conclude significant difference between posteriors with and without TESS
pvalthrs = 1e-6

indxcomp = np.arange(3)

# list of rtyp
# 0 : wout
# 1 : with
# 2 : only
# list of comp
# wout with
# wout only
# with only
indxrtypfrst = [0, 0, 1]
indxrtypseco = [1, 2, 2]

liststrgrtyp = ['wout', 'with', 'only'] 
numbrtyp = len(liststrgrtyp)
indxrtyp = np.arange(numbrtyp)

lpos = [[] for i in indxrtyp]
chan = [[] for i in indxrtyp]
lablpara = [[] for i in indxrtyp]
numbwalk = [[] for i in indxrtyp]
numbswep = [[] for i in indxrtyp]
numbsamp = [[] for i in indxrtyp]
numbburn = [[] for i in indxrtyp]
factthin = [[] for i in indxrtyp]
numbpara = [[] for i in indxrtyp]

pathdataplan = pathdata + strgplan+ '/'

for i, strgrtyp in enumerate(liststrgrtyp):
    pathdata = pathdataplan + 'allesfit_%s/allesfit_%stess_ns/' % (strgkind, strgrtyp)
    print('Reading from %s...' % pathdata)
    config.init(pathdata)
    lablpara[i] = config.BASEMENT.fitlabels

    pathsave = pathdata + 'results/mcmc_save.h5'
    if False and not os.path.exists(pathsave):
        # sample from the posterior excluding the TESS data
        print('Calling allesfitter to fit the data...')
        allesfitter.mcmc_fit(pathdata)

    # read the chain
    ## MCMC
    #emceobjt = emcee.backends.HDFBackend(pathsave, read_only=True)
    #chan[i] = emceobjt.get_chain()
    #lpos[i] = emceobjt.get_log_prob()
    
    ## Nested sampling
    fileobjt = open(pathdata + 'results/save_ns.pickle', 'rb')
    objtrest = pickle.load(fileobjt)
    weig = np.exp(objtrest['logwt'] - objtrest['logz'][-1])
    chan[i] = dyutils.resample_equal(objtrest.samples, weig)    
    lpos[i] = np.zeros(chan[i].shape[0])

    # parse configuration
    numbswep[i] = config.BASEMENT.settings['mcmc_total_steps']
    numbburn[i] = config.BASEMENT.settings['mcmc_burn_steps']
    factthin[i] = config.BASEMENT.settings['mcmc_thin_by']
    numbpara[i] = config.BASEMENT.ndim

    numbwalk[i] = chan[i].shape[1]
    numbswep[i] = chan[i].shape[0] * factthin
    numbsamp[i] = lpos[i].size
    chan[i] = chan[i].reshape((-1, numbpara[i]))
    print('Found %d samples and %d parameters...' % (chan[i].shape[0], chan[i].shape[1]))

lablparacomp = [[] for u in indxcomp]
for u in indxcomp:

    lablparacomp[u] = list(set(lablpara[indxrtypfrst[u]]).intersection(lablpara[indxrtypseco[u]]))

    # post-processing
    ## calculate the KS test statistic between the posteriors
    numbparacomp = len(lablparacomp[u])
    pval = np.empty(numbparacomp)
    for j in range(numbparacomp):
        kosm, pval[j] = stats.ks_2samp(chan[indxrtypfrst[u]][:, j], chan[indxrtypseco[u]][:, j])

    ## find the list of parameters whose posterior with and without TESS are unlikely to be drawn from the same distribution
    indxparagood = np.where(pval < pvalthrs)[0]
    if indxparagood.size > 0:

        figr, axis = plt.subplots(figsize=(12, 5))
        indxparacomp = np.arange(numbparacomp)
        axis.plot(indxparacomp, pval, ls='', marker='o')
        axis.plot(indxparacomp[indxparagood], pval[indxparagood], ls='', marker='o', color='r')
        axis.set_yscale('log')
        axis.set_xticks(indxparacomp)
        axis.set_xticklabels(lablparacomp[u])
        if u == 0:
            axis.set_title('Posteriors with TESS vs. without TESS')
        if u == 1:
            axis.set_title('Posteriors without TESS vs. only TESS')
        if u == 2:
            axis.set_title('Posteriors with TESS vs. only TESS')

        axis.axhline(pvalthrs, ls='--', color='black', alpha=0.3)
        plt.tight_layout()
        path = pathdata + 'postproc/%s_kosm_com%d_%s.png' % (strgtimestmp, u, strgplan)
        print('Writing to %s...' % path)
        figr.savefig(path)
        plt.close()

indxrtypmaxm = 1

lablparatotl = np.unique(np.concatenate(lablpara))

for l, lablparatemp in enumerate(lablparatotl):
    figr, axis = plt.subplots()
    numbbins = 50

    listindxrtypcomp = []
    for i in indxrtyp:
        if lablparatemp in lablpara[i]:
            listindxrtypcomp.append(i)

    figr, axis = plt.subplots()
    chanlist = []
    for i in listindxrtypcomp:
        m = np.where(lablpara[i] == lablparatemp)[0][0]

        if lablparatemp == '$T_{0;b}$':
            offs = np.mean(chan[i][:, m])
            chantemp = chan[i][:, m] - offs
            print('Subtracting %g from T_b for TESS...' % offs)
        else:
            chantemp = chan[i][:, m]

        chanlist.append(chantemp)
    axis.violinplot(chanlist, showmedians=True, showextrema=False)

    if listindxrtypcomp == [0, 1]:
        labltick = ['w/o TESS', 'w/ TESS']
    if listindxrtypcomp == [0, 2]:
        labltick = ['w/o TESS', 'only TESS']
    if listindxrtypcomp == [1, 2]:
        labltick = ['w/ TESS', 'only TESS']
    if listindxrtypcomp == [0, 1, 2]:
        labltick = ['w/o TESS', 'w/ TESS', 'only TESS']
    valutick = np.arange(len(labltick)) + 1

    axis.set_xticks(valutick)
    axis.set_xticklabels(labltick)
    axis.set_ylabel(lablparatemp)
    plt.tight_layout()

    path = pathdata + 'postproc/%s_viol_pr%02d_%s.png' % (strgtimestmp, l, strgplan)
    print('Writing to %s...' % path)
    figr.savefig(path)
    plt.close()

# summary violin plot 
figr, axis = plt.subplots()
chanlist = []
for l, lablparatemp in enumerate(['']):
    chanlist.append()
axis.violinplot(chanlist, showmedians=True, showextrema=False)

axis.set_xticks(valutick)
axis.set_xticklabels(labltick)
axis.set_ylabel(lablparatemp)
plt.tight_layout()

path = pathdata + 'postproc/%s_viol_pr%02d_%s.png' % (strgtimestmp, l, strgplan)
print('Writing to %s...' % path)
figr.savefig(path)
plt.close()





"""
Utility functions for post processing

Tansu Daylan
MIT Kavli Institute, Cambridge, MA, 02109, US
tansu.daylan@gmail.com
www.tansudaylan.com
"""

def demo_kosm(gdat):

    '''
    Demonstrates Kolmogorov-Smirnov test statistics
    '''

    path = pathdata + 'plt.png'

    if not os.path.exists(path):
        numbsamp = 1000
        numbpara = 4
        indxpara = np.arange(numbpara)

        shft = 1.5 * np.random.randn(numbpara)
        shrk = 0.5 * np.random.randn(numbpara) + 1.

        chan = np.random.randn(numbpara * numbsamp).reshape((numbsamp, numbpara))
        for k in indxpara:
            chan[:, k] = shft[k] + np.random.randn(numbsamp) / shrk[k]

        strgtitl = ''
        figr, axis = plt.subplots()
        for k in indxpara:
            plt.hist(chan[:, k], alpha=0.4, lw=10, histtype='stepfilled')
            if k != 0:
                kosm, pval = stats.ks_2samp(chan[:, 0], chan[:, k])
                strgtitl += ' %.5g' % pval
        axis.set_title(strgtitl)
        plt.tight_layout()
        figr.savefig(path)
        plt.close()


def retr_listexop(gdat):

    pass

    # temp
    #coords = SkyCoord(['124.532 -68.313', '42.42, -42.42'], unit='deg')
    #df = gts.get_time_on_silicon(coords)
    #print 'Time on silicon'
    #print 'df'
    #print df






#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Fri Oct  5 01:03:21 2018

@author:
Maximilian N. Günther
MIT Kavli Institute for Astrophysics and Space Research, 
Massachusetts Institute of Technology,
77 Massachusetts Avenue,
Cambridge, MA 02109, 
USA
Email: maxgue@mit.edu
Web: www.mnguenther.com
"""

from __future__ import print_function, division, absolute_import

#::: modules
import numpy as np
import os
import emcee
from multiprocessing import Pool
from contextlib import closing

#::: warnings
import warnings
warnings.filterwarnings('ignore', category=np.VisibleDeprecationWarning) 
warnings.filterwarnings('ignore', category=np.RankWarning) 

#::: allesfitter modules
from . import config
from .computer import update_params,\
                      calculate_lnlike
from .general_output import logprint
from .mcmc_output import print_autocorr



###############################################################################
#::: MCMC log likelihood
###############################################################################
def mcmc_lnlike(theta):
    
    params = update_params(theta)
    lnlike = 0
    
    for inst in config.BASEMENT.settings['inst_phot']:
        lnlike += calculate_lnlike(params, inst, 'flux')
    
    for inst in config.BASEMENT.settings['inst_rv']:
        lnlike += calculate_lnlike(params, inst, 'rv')

    return lnlike

    
        
###############################################################################
#::: MCMC log prior
###############################################################################
def mcmc_lnprior(theta):
    '''
    bounds has to be list of len(theta), containing tuples of form
    ('none'), ('uniform', lower bound, upper bound), or ('normal', mean, std)
    '''
    lnp = 0.        
    
    for th, b in zip(theta, config.BASEMENT.bounds):
        if b[0] == 'uniform':
            if not (b[1] <= th <= b[2]): return -np.inf
        elif b[0] == 'normal':
            lnp += np.log( 1./(np.sqrt(2*np.pi) * b[2]) * np.exp( - (th - b[1])**2 / (2.*b[2]**2) ) )
        else:
            raise ValueError('Bounds have to be "uniform" or "normal". Input from "params.csv" was "'+b[0]+'".')
    return lnp



###############################################################################
#::: MCMC log probability
###############################################################################    
def mcmc_lnprob(theta):
    '''
    has to be top-level for  for multiprocessing pickle
    '''
    lp = mcmc_lnprior(theta)
    if not np.isfinite(lp):
        return -np.inf
    else:
        try:
            ln = mcmc_lnlike(theta)
            return lp + ln
        except:
            return -np.inf
        


###########################################################################
#::: MCMC fit
###########################################################################
def mcmc_fit(datadir):
    
    #::: init
    config.init(datadir)
    
    
    continue_old_run = False
    if os.path.exists(os.path.join(config.BASEMENT.outdir,'mcmc_save.h5')):
        overwrite = str(input(os.path.join(config.BASEMENT.outdir,'mcmc_save.h5')+\
                              ' already exists.\n'+\
                              'What do you want to do?\n'+\
                              '1 : overwrite the save file\n'+\
                              '2 : append to the save file\n'+\
                              '3 : abort\n'))
        if (overwrite == '1'):
            continue_old_run = False
        elif (overwrite == '2'):
            continue_old_run = True
        else:
            raise ValueError('User aborted operation.')
            
    
    #::: set up a backend
    backend = emcee.backends.HDFBackend(os.path.join(config.BASEMENT.outdir,'mcmc_save.h5'))
    
    
    #::: continue on the backend / reset the backend
    if not continue_old_run:
        backend.reset(config.BASEMENT.settings['mcmc_nwalkers'], config.BASEMENT.ndim)
    
    
    #::: helper fct
    def run_mcmc(sampler):
        
        #::: set initial walker positions
        if continue_old_run:
            p0 = backend.get_chain()[-1,:,:]
            already_completed_steps = backend.get_chain().shape[0] * config.BASEMENT.settings['mcmc_thin_by']
        else:
            p0 = config.BASEMENT.theta_0 + config.BASEMENT.init_err * np.random.randn(config.BASEMENT.settings['mcmc_nwalkers'], config.BASEMENT.ndim)
            already_completed_steps = 0
        
        #::: make sure the inital positions are within the limits
        for i, b in enumerate(config.BASEMENT.bounds):
            if b[0] == 'uniform':
                p0[:,i] = np.clip(p0[:,i], b[1], b[2]) 
        
        #::: run the sampler        
        sampler.run_mcmc(p0,
                         (config.BASEMENT.settings['mcmc_total_steps'] - already_completed_steps)/config.BASEMENT.settings['mcmc_thin_by'], 
                         thin_by=config.BASEMENT.settings['mcmc_thin_by'], 
                         progress=True)
        
        return sampler


    #::: Run
    logprint("\nRunning MCMC...")
    logprint('--------------------------')
    if config.BASEMENT.settings['multiprocess']:
         with closing(Pool(processes=(config.BASEMENT.settings['multiprocess_cores']))) as pool: #multiprocessing
#        with closing(Pool(cpu_count()-1)) as pool: #pathos
            logprint('\nRunning on', config.BASEMENT.settings['multiprocess_cores'], 'CPUs.')
            sampler = emcee.EnsembleSampler(config.BASEMENT.settings['mcmc_nwalkers'], 
                                            config.BASEMENT.ndim, 
                                            mcmc_lnprob,
                                            pool=pool, 
                                            backend=backend)
            sampler = run_mcmc(sampler)
    else:
        sampler = emcee.EnsembleSampler(config.BASEMENT.settings['mcmc_nwalkers'],
                                        config.BASEMENT.ndim,
                                        mcmc_lnprob,
                                        backend=backend)
        sampler = run_mcmc(sampler)
    

    #::: Check performance and convergence
    logprint('\nAcceptance fractions:')
    logprint('--------------------------')
    logprint(sampler.acceptance_fraction)
    
    print_autocorr(sampler)
    
    


#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Fri Sep 28 15:19:30 2018

@author:
Maximilian N. Günther
MIT Kavli Institute for Astrophysics and Space Research, 
Massachusetts Institute of Technology,
77 Massachusetts Avenue,
Cambridge, MA 02109, 
USA
Email: maxgue@mit.edu
Web: www.mnguenther.com
"""

from __future__ import print_function, division, absolute_import

#::: plotting settings
import seaborn as sns
sns.set(context='paper', style='ticks', palette='deep', font='sans-serif', font_scale=1.5, color_codes=True)
sns.set_style({"xtick.direction": "in","ytick.direction": "in"})
sns.set_context(rc={'lines.markeredgewidth': 1})

#::: modules
import os
#import collections
import numpy as np
import matplotlib.pyplot as plt
import pickle
from corner import corner
from tqdm import tqdm 
from astropy.constants import M_earth, M_jup, M_sun, R_earth, R_jup, R_sun, au

#::: allesfitter modules
from . import config
from .utils import latex_printer
from .general_output import logprint
from .priors.simulate_PDF import simulate_PDF
from .computer import update_params, calculate_model

#::: constants
#M_earth = 5.9742e+24 	#kg 	Earth mass
#M_jup   = 1.8987e+27 	#kg 	Jupiter mass
#M_sun   = 1.9891e+30 	#kg 	Solar mass
#R_earth = 6378136      #m 	Earth equatorial radius
#R_jup   = 71492000 	#m 	Jupiter equatorial radius
#R_sun   = 695508000 	#m 	Solar radius




def derive(samples, mode):
    '''
    Derives parameter of the system using Winn 2010
    
    Input:
    ------
    samples : array
        samples from the mcmc or nested sampling
    mode : str
        'mcmc' or 'ns'
        
    Returns:
    --------
    derived_samples : dict 
        with keys 'i', 'R1a', 'R2a', 'k', 'depth_undiluted', 'b_tra', 'b_occ', 'Ttot', 'Tfull'
        each key contains all the samples derived from the MCMC samples 
        (not mean values, but pure samples!)
        i = inclination 
        R1a = R1/a, radius companion over semiamplitude
        R2a = R2/a, radius star over semiamplitude
        Ttot = T_{1-4}, total transit width 
        Tfull = T_{2-3}, full transit width
        
    Output:
    -------
    latex table of results
    corner plot of derived values posteriors
    '''
    
    N_samples = samples.shape[0]
    

    ###############################################################################
    #::: stellar 'posteriors'
    ###############################################################################
    buf = np.genfromtxt( os.path.join(config.BASEMENT.datadir,'params_star.csv'), delimiter=',', names=True, dtype=None, encoding='utf-8', comments='#' )
    star = {}
    star['R_star'] = simulate_PDF(buf['R_star'], buf['R_star_lerr'], buf['R_star_uerr'], size=N_samples, plot=False)
    star['M_star'] = simulate_PDF(buf['M_star'], buf['M_star_lerr'], buf['M_star_uerr'], size=N_samples, plot=False)
    star['Teff_star'] = simulate_PDF(buf['Teff_star'], buf['Teff_star_lerr'], buf['Teff_star_uerr'], size=N_samples, plot=False)
    
    
    
    ###############################################################################
    #::: derive all the params
    ###############################################################################
    companions = config.BASEMENT.settings['companions_all']
    
    def get_params(key):
        ind = np.where(config.BASEMENT.fitkeys==key)[0]
        if len(ind)==1: return samples[:,ind].flatten() #if it was fitted for
        else: 
            try:
                return config.BASEMENT.params[key] #else take the input value
            except KeyError:
                return np.nan
        
    def sin_d(alpha): return np.sin(np.deg2rad(alpha))
    def cos_d(alpha): return np.cos(np.deg2rad(alpha))
    def arcsin_d(x): return np.rad2deg(np.arcsin(x))
    def arccos_d(x): return np.rad2deg(np.arccos(x))

    derived_samples = {}
    for companion in companions:
        
        #::: radii
        derived_samples[companion+'_R_star/a'] = get_params(companion+'_rsuma') / (1. + get_params(companion+'_rr'))
        derived_samples[companion+'_R_companion/a'] = get_params(companion+'_rsuma') * get_params(companion+'_rr') / (1. + get_params(companion+'_rr'))
        derived_samples[companion+'_R_companion_(R_earth)'] = star['R_star'] * get_params(companion+'_rr') * R_sun.value / R_earth.value #in R_earth
        derived_samples[companion+'_R_companion_(R_jup)'] = star['R_star'] * get_params(companion+'_rr') * R_sun.value / R_jup.value #in R_jup

        derived_samples[companion+'_depth_undiluted'] = 1e3*get_params(companion+'_rr')**2 #in mmag


        #::: orbit
        derived_samples[companion+'_a_(R_sun)'] = star['R_star'] / derived_samples[companion+'_R_star/a']   
        derived_samples[companion+'_a_(AU)'] = derived_samples[companion+'_a_(R_sun)'] * R_sun.value/au.value
        derived_samples[companion+'_i'] = arccos_d(get_params(companion+'_cosi')) #in deg
        derived_samples[companion+'_e'] = get_params(companion+'_f_s')**2 + get_params(companion+'_f_c')**2
        derived_samples[companion+'_e_sinw'] = get_params(companion+'_f_s') * np.sqrt(derived_samples[companion+'_e'])
        derived_samples[companion+'_e_cosw'] = get_params(companion+'_f_c') * np.sqrt(derived_samples[companion+'_e'])
        derived_samples[companion+'_w'] = arccos_d( get_params(companion+'_f_c') / np.sqrt(derived_samples[companion+'_e']) ) #in deg, from 0 to 180
        
        
        #::: mass
        if companion+'_K' in config.BASEMENT.params:
            a_1 = 0.019771142 * get_params(companion+'_K') * get_params(companion+'_period') * np.sqrt(1. - derived_samples[companion+'_e']**2)/sin_d(derived_samples[companion+'_i'])
    #        derived_samples[companion+'_a_rv'] = (1.+1./ellc_params[companion+'_q'])*a_1
            derived_samples[companion+'_q'] = 1./(( derived_samples[companion+'_a_(R_sun)'] / a_1 ) - 1.)
            derived_samples[companion+'_M_companion_(M_earth)'] = derived_samples[companion+'_q'] * star['M_star'] * M_sun.value / M_earth.value #in M_earth
            derived_samples[companion+'_M_companion_(M_jup)'] = derived_samples[companion+'_q'] * star['M_star'] * M_sun.value / M_jup.value #in M_jup
        else:
            derived_samples[companion+'_M_companion'] = None
            
            
        #::: surface gravity
        #::: from Southworth+ 2007
        if companion+'_K' in config.BASEMENT.params:
            derived_samples[companion+'_g'] = 2*np.pi / get_params(companion+'_period') * (1.-derived_samples[companion+'_e']**2)**0.5 * get_params(companion+'_K') / (derived_samples[companion+'_R_companion/a']**2 * sin_d(derived_samples[companion+'_i']))
        else:
            derived_samples[companion+'_g'] = None        
            
            
        #::: time of occultation    
        if config.BASEMENT.settings['secondary_eclipse'] is True:
            derived_samples[companion+'_dt_occ'] = get_params(companion+'_period')/2. * (1. + 4./np.pi * derived_samples[companion+'_e'] * cos_d(derived_samples[companion+'_w'])  ) #approximation
        else:
            derived_samples[companion+'_dt_occ'] = None
        
        
        #::: impact params
        derived_samples[companion+'_b_tra'] = (1./derived_samples[companion+'_R_star/a']) * get_params(companion+'_cosi') * ( (1.-derived_samples[companion+'_e']**2) / ( 1.+derived_samples[companion+'_e']*sin_d(derived_samples[companion+'_w']) ) )
         
        if config.BASEMENT.settings['secondary_eclipse'] is True:
            derived_samples[companion+'_b_occ'] = (1./derived_samples[companion+'_R_star/a']) * get_params(companion+'_cosi') * ( (1.-derived_samples[companion+'_e']**2) / ( 1.-derived_samples[companion+'_e']*sin_d(derived_samples[companion+'_w']) ) )
        else:
            derived_samples[companion+'_b_occ'] = None
        
        #::: transit duration 
        derived_samples[companion+'_T_tra_tot'] = get_params(companion+'_period')/np.pi *24.  \
                                  * np.arcsin( derived_samples[companion+'_R_star/a'] \
                                             * np.sqrt( (1.+get_params(companion+'_rr'))**2 - derived_samples[companion+'_b_tra']**2 )\
                                             / sin_d(derived_samples[companion+'_i']) ) #in h
        derived_samples[companion+'_T_tra_full'] = get_params(companion+'_period')/np.pi *24.  \
                                  * np.arcsin( derived_samples[companion+'_R_star/a'] \
                                             * np.sqrt( (1.-get_params(companion+'_rr'))**2 - derived_samples[companion+'_b_tra']**2 )\
                                             / sin_d(derived_samples[companion+'_i']) ) #in h
                                  

        #::: secondary eclipse / occultation depth (per inst)
        for inst in config.BASEMENT.settings['inst_phot']:
            derived_samples[companion+'_depth_occ_max_diluted_'+inst]  = np.zeros(N_samples)*np.nan
            derived_samples[companion+'_depth_occ_norm_diluted_'+inst] = np.zeros(N_samples)*np.nan
            
            if config.BASEMENT.settings['secondary_eclipse'] is True:
                    logprint('Deriving occultation depths from model curves...')
                    xx = np.linspace( 0.25, 0.75, 500)
                    for i in tqdm( range(N_samples) ):
                        s = samples[i,:]
                        p = update_params(s, phased=True)
                        model = calculate_model(p, inst, 'flux', xx=xx) #evaluated on xx (!)
                        derived_samples[companion+'_depth_occ_max_diluted_'+inst][i] = ( np.max(model) - np.min(model) ) * 1e6 #in ppm
                        derived_samples[companion+'_depth_occ_norm_diluted_'+inst][i] = ( 1. - np.min(model) ) * 1e6 #in ppm
                    
    
        #::: undiluted (not per companions; per inst)
        for inst in config.BASEMENT.settings['inst_phot']:
            dil = get_params('light_3_'+inst)
        #        if np.mean(dil)<0.5: dil = 1-dil
            derived_samples[companion+'_depth_diluted_'+inst] = derived_samples[companion+'_depth_undiluted'] * (1. - dil) #in mmag
            derived_samples[companion+'_depth_occ_max_undiluted_'+inst] = derived_samples[companion+'_depth_occ_max_diluted_'+inst] / (1. - dil) #in ppm
            derived_samples[companion+'_depth_occ_norm_undiluted_'+inst] = derived_samples[companion+'_depth_occ_norm_diluted_'+inst] / (1. - dil) #in ppm
        
        
        #::: equilibirum temperature
        #::: currently assumes Albedo of 0.3 and Emissivity of 1
        albedo = 0.3
        emissivity = 1.
        derived_samples[companion+'_Teq'] = star['Teff_star']  * ( (1.-albedo)/emissivity )**0.25 * np.sqrt(derived_samples[companion+'_R_star/a'] / 2.)
        
        
        #::: period ratios (for ressonance studies)
        if len(companions)>1:
            for other_companion in companions:
                if other_companion is not companion:
                    derived_samples[companion+'_period/'+other_companion+'_period'] = get_params(companion+'_period') / get_params(other_companion+'_period')
        
        
    
    ###############################################################################
    #::: write keys for output
    ###############################################################################
    names = []
    labels = []
    for companion in companions:
            
        names.append( companion+'_R_star/a' )
        labels.append( '$R_\star/a_\mathrm{'+companion+'}$' )
        
        names.append( companion+'_R_companion/a'  )
        labels.append( '$R_\mathrm{'+companion+'}/a_\mathrm{'+companion+'}$' )
        
        names.append( companion+'_R_companion_(R_earth)' )
        labels.append( '$R_\mathrm{'+companion+'}$ ($\mathrm{R_{\oplus}}$)' )
        
        names.append( companion+'_R_companion_(R_jup)' )
        labels.append( '$R_\mathrm{'+companion+'}$ ($\mathrm{R_{jup}}$)' )
        
        names.append( companion+'_a_(R_sun)' )
        labels.append( '$a_\mathrm{'+companion+'}$ ($\mathrm{R_{\odot}}$)' )
        
        names.append( companion+'_a_(AU)' )
        labels.append( '$a_\mathrm{'+companion+'}$ (AU)' )
        
        names.append( companion+'_i' )
        labels.append( '$i_\mathrm{'+companion+'}$ (deg)' )
        
        names.append( companion+'_e' )
        labels.append( '$e_\mathrm{'+companion+'}$' )
        
        names.append( companion+'_w' )
        labels.append( '$w_\mathrm{'+companion+'}$ (deg)' )
        
        names.append( companion+'_M_companion_(M_earth)' )
        labels.append( '$M_\mathrm{'+companion+'}$ ($\mathrm{M_{\oplus}}$)' )
        
        names.append( companion+'_M_companion_(M_jup)' )
        labels.append( '$M_\mathrm{'+companion+'}$ ($\mathrm{M_{jup}}$)' )
        
        names.append( companion+'_b_tra' )
        labels.append( '$b_\mathrm{tra;'+companion+'}$' )
        
        names.append( companion+'_b_occ'  )
        labels.append( '$b_\mathrm{occ;'+companion+'}$' )
        
        names.append( companion+'_T_tra_tot'  )
        labels.append( '$T_\mathrm{tot;'+companion+'}$ (h)' )
        
        names.append( companion+'_T_tra_full' )
        labels.append( '$T_\mathrm{full;'+companion+'}$ (h)' )
        
        names.append( companion+'_Teq' )
        labels.append( '$T_\mathrm{eq;'+companion+'}$ (K)' )
        
        names.append( companion+'_depth_undiluted' )
        labels.append( '$\delta_\mathrm{undil; '+companion+'}$ (mmag)' )
            
        
        for inst in config.BASEMENT.settings['inst_phot']:
            
            names.append( companion+'_depth_diluted_'+inst )
            labels.append( '$\delta_\mathrm{dil; '+inst+'}$ (mmag)' )
            
            names.append( companion+'_depth_occ_max_undiluted_'+inst )
            labels.append( '$\delta_\mathrm{occ; max; undil; '+inst+'}$ (mmag)' )
            
            names.append( companion+'_depth_occ_norm_undiluted_'+inst )
            labels.append( '$\delta_\mathrm{occ; norm; undil; '+inst+'}$ (mmag)' )
            
            names.append( companion+'_depth_occ_max_diluted_'+inst )
            labels.append( '$\delta_\mathrm{occ; max; dil; '+inst+'}$ (mmag)' )
            
            names.append( companion+'_depth_occ_norm_diluted_'+inst )
            labels.append( '$\delta_\mathrm{occ; norm; dil; '+inst+'}$ (mmag)' )
            
            
        #::: period ratios (for ressonance studies)
        if len(companions)>1:
            for other_companion in companions:
                if other_companion is not companion:
                    names.append( companion+'_period/'+other_companion+'_period' )
                    labels.append( '$P_\mathrm{'+companion+'} / P_\mathrm{'+other_companion+'}$' )
                    

        
            
    ###############################################################################
    #::: delete pointless values
    ###############################################################################
    ind_good = []
    for i,name in enumerate(names):
        if isinstance(derived_samples[name], np.ndarray) and not any(np.isnan(derived_samples[name])) and not all(np.array(derived_samples[name])==0):
            ind_good.append(i)
            
    names = [ names[i] for i in ind_good ]
    labels = [ labels[i] for i in ind_good ]
    
    
            
    ###############################################################################
    #::: save all in pickle
    ###############################################################################
    pickle.dump(derived_samples, open(os.path.join(config.BASEMENT.outdir,mode+'_derived_samples.pickle'),'wb'))
    
    
    
    ###############################################################################
    #::: save txt & latex table & latex commands
    ###############################################################################
    with open(os.path.join(config.BASEMENT.outdir,mode+'_derived_table.csv'),'w') as outfile,\
         open(os.path.join(config.BASEMENT.outdir,mode+'_derived_latex_table.txt'),'w') as f,\
         open(os.path.join(config.BASEMENT.outdir,mode+'_derived_latex_cmd.txt'),'w') as f_cmd:
             
        outfile.write('#property,value,lower_error,upper_error,source\n')
        
        f.write('Property & Value & Source \\\\ \n')
        f.write('\\hline \n')
        f.write('\\multicolumn{4}{c}{\\textit{Derived parameters}} \\\\ \n')
        f.write('\\hline \n')
        
        for name,label in zip(names, labels):
            ll, median, ul = np.percentile(derived_samples[name], [15.865, 50., 84.135])
            outfile.write( str(label)+','+str(median)+','+str(median-ll)+','+str(ul-median)+',derived\n' )
            
            value = latex_printer.round_tex(median, median-ll, ul-median)
            f.write( label + ' & $' + value + '$ & derived \\\\ \n' )
            
            simplename = name.replace("_", "").replace("/", "over").replace("(", "").replace(")", "").replace("1", "one").replace("2", "two")
            f_cmd.write('\\newcommand{\\'+simplename+'}{$'+value+'$} %'+label+' = '+value+'\n')
            
    logprint('\nSaved '+mode+'_derived_results.csv, '+mode+'_derived_latex_table.txt, and '+mode+'_derived_latex_cmd.txt')
    
        
        
    ###############################################################################
    #::: plot corner
    ###############################################################################
    x = np.column_stack([ derived_samples[name] for name in names ])
    fig = corner(x,
                 range = [0.999]*len(names),
                 labels = names,
                 quantiles=[0.15865, 0.5, 0.84135],
                 show_titles=True, title_kwargs={"fontsize": 14})
    fig.savefig( os.path.join(config.BASEMENT.outdir,mode+'_derived_corner.png'), dpi=100, bbox_inches='tight' )
    plt.close(fig)
    
    logprint('\nSaved '+mode+'_derived_corner.jpg')
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sat Feb  2 16:06:09 2019

@author:
Maximilian N. Günther
MIT Kavli Institute for Astrophysics and Space Research, 
Massachusetts Institute of Technology,
77 Massachusetts Avenue,
Cambridge, MA 02109, 
USA
Email: maxgue@mit.edu
Web: www.mnguenther.com
"""

from __future__ import print_function, division, absolute_import

#::: plotting settings
import seaborn as sns
sns.set(context='paper', style='ticks', palette='deep', font='sans-serif', font_scale=1.5, color_codes=True)
sns.set_style({"xtick.direction": "in","ytick.direction": "in"})
sns.set_context(rc={'lines.markeredgewidth': 1})

#::: modules
import numpy as np
import matplotlib.pyplot as plt
from matplotlib import gridspec
from matplotlib.patches import Circle
import matplotlib.ticker as plticker
from matplotlib.collections import LineCollection
from matplotlib.colors import LinearSegmentedColormap
import random
import warnings
try:
    import rebound
    from rebound.particle import Particle
except:
    warnings.warn('Module "rebound" could not be imported. Orbital plots are not available.')
from itertools import cycle

#::: allesfitter codes
from allesfitter import config
from allesfitter.exoworlds_rdx.lightcurves.index_transits import get_first_epoch
from allesfitter.exoworlds_rdx.lightcurves.lightcurve_tools import calc_phase




def OrbitPlot(sim, figsize=None, lim=None, limz=None, Narc=100, xlabel='x', ylabel='y', zlabel='z', color=False, periastron=False, trails=True, show_orbit=True, lw=1., glow=False, slices=False, plotparticles=[], primary=None, fancy=False):
    """
    Convenience function for plotting instantaneous orbits.

    Parameters
    ----------
    slices          : bool, optional
        Plot all three slices if set to True. Default is False and plots orbits only in the xy plane.
    figsize         : tuple of float, optional
        Tuple defining the figure size (default: (5,5))
    lim             : float, optional           
        Limit for axes (default: None = automatically determined)
    limz            : float, optional           
        Limit for z axis, only used if slices=True (default: None = automatically determined)
    unitlabel       : str, optional          
        String describing the units, shown on axis labels (default: None)
    color           : bool, str or list, optional            
        By default plots in black. If set to True, plots using REBOUND color cycle. If a string or list of strings, e.g. ['red', 'cyan'], will cycle between passed colors.
    periastron  : bool, optional            
        Draw a marker at periastron (default: False)
    trails          : bool, optional            
        Draw trails instead of solid lines (default: False)
    show_orbit      : bool, optional
        Draw orbit trails/lines (default: True)
    lw              : float, optional           
        Linewidth (default: 1.)
    glow            : bool (default: False)
        Make lines glow
    fancy           : bool (default: False)
        Changes various settings to create a fancy looking plot
    plotparticles   : list, optional
        List of particles to plot. Can be a list of any valid keys for accessing sim.particles, i.e., integer indices or hashes (default: plot all particles)
    primary         : rebound.Particle, optional
        Pimrary to use for the osculating orbit (default: Jacobi center of mass)

    Returns
    -------
    fig
        A matplotlib figure

    Examples
    --------
    The following example illustrates a typical use case.

    >>> sim = rebound.Simulation()
    >>> sim.add(m=1)
    >>> sim.add(a=1)
    >>> fig = rebound.OrbitPlot(sim)
    >>> fig.savefig("image.png") # save figure to file
    >>> fig.show() # show figure on screen

    """
    if slices:
        if figsize is None:
            figsize = (8,8)
        fig, ax = plt.subplots(2, 2, figsize=figsize)
        gs = gridspec.GridSpec(2, 2, width_ratios=[3., 2.], height_ratios=[2.,3.],wspace=0., hspace=0.) 
        OrbitPlotOneSlice(sim, plt.subplot(gs[2]), lim=lim, Narc=Narc, color=color, periastron=periastron, trails=trails, show_orbit=show_orbit, lw=lw, axes="xy",fancy=fancy, plotparticles=plotparticles, primary=primary, glow=glow)
        OrbitPlotOneSlice(sim, plt.subplot(gs[3]), lim=lim, limz=limz, Narc=Narc, color=color, periastron=periastron, trails=trails, show_orbit=show_orbit, lw=lw,fancy=fancy, axes="zy", plotparticles=plotparticles, primary=primary, glow=glow)
        OrbitPlotOneSlice(sim, plt.subplot(gs[0]), lim=lim, limz=limz, Narc=Narc, color=color, periastron=periastron, trails=trails, show_orbit=show_orbit, lw=lw,fancy=fancy, axes="xz", plotparticles=plotparticles, primary=primary, glow=glow)
        plt.subplot(gs[2]).set_xlabel(xlabel)
        plt.subplot(gs[2]).set_ylabel(ylabel)
      
        plt.setp(plt.subplot(gs[0]).get_xticklabels(), visible=False)
        plt.subplot(gs[0]).set_ylabel(zlabel)
        
        plt.subplot(gs[3]).set_xlabel(zlabel)
        plt.setp(plt.subplot(gs[3]).get_yticklabels(), visible=False)
    else:
        if figsize is None:
            figsize = (5,5)
        fig, ax = plt.subplots(1, 1, figsize=figsize)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        OrbitPlotOneSlice(sim, ax, lim=lim, Narc=Narc, color=color, periastron=periastron, trails=trails, show_orbit=show_orbit, lw=lw,fancy=fancy, plotparticles=plotparticles, primary=primary, glow=glow)
    return fig

def get_color(color):
    """
    Takes a string for a color name defined in matplotlib and returns of a 3-tuple of RGB values.
    Will simply return passed value if it's a tuple of length three.

    Parameters
    ----------
    color   : str
        Name of matplotlib color to calculate RGB values for.
    """

    if isinstance(color, tuple) and len(color) == 3: # already a tuple of RGB values
        return color

    hexcolor = sns.colors.xkcd_rgb[color]

    hexcolor = hexcolor.lstrip('#')
    lv = len(hexcolor)
    return tuple(int(hexcolor[i:i + lv // 3], 16)/255. for i in range(0, lv, lv // 3)) # tuple of rgb values



def fading_line(x, y, color='black', alpha_initial=1., alpha_final=0., glow=False, **kwargs):
    """
    Returns a matplotlib LineCollection connecting the points in the x and y lists, with a single color and alpha varying from alpha_initial to alpha_final along the line.
    Can pass any kwargs you can pass to LineCollection, like linewidgth.

    Parameters
    ----------
    x       : list or array of floats for the positions on the (plot's) x axis
    y       : list or array of floats for the positions on the (plot's) y axis
    color   : matplotlib color for the line. Can also pass a 3-tuple of RGB values (default: 'black')
    alpha_initial:  Limiting value of alpha to use at the beginning of the arrays.
    alpha_final:    Limiting value of alpha to use at the end of the arrays.
    """
    if glow:
        glow = False
        kwargs["lw"] = 1
        fl1 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        kwargs["lw"] = 2
        alpha_initial *= 0.5
        alpha_final *= 0.5
        fl2 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        kwargs["lw"] = 6
        alpha_initial *= 0.5
        alpha_final *= 0.5
        fl3 = fading_line(x, y, color, alpha_initial, alpha_final, glow=False, **kwargs)
        return [fl3,fl2,fl1]

    color = get_color(color)
    cdict = {'red': ((0.,color[0],color[0]),(1.,color[0],color[0])),
             'green': ((0.,color[1],color[1]),(1.,color[1],color[1])),
             'blue': ((0.,color[2],color[2]),(1.,color[2],color[2])),
             'alpha': ((0.,alpha_initial, alpha_initial), (1., alpha_final, alpha_final))}
    
    Npts = len(x)
    if len(y) != Npts:
        raise AttributeError("x and y must have same dimension.")
   
    segments = np.zeros((Npts-1,2,2))
    segments[0][0] = [x[0], y[0]]
    for i in range(1,Npts-1):
        pt = [x[i], y[i]]
        segments[i-1][1] = pt
        segments[i][0] = pt 
    segments[-1][1] = [x[-1], y[-1]]

    individual_cm = LinearSegmentedColormap('indv1', cdict)
    lc = LineCollection(segments, cmap=individual_cm, **kwargs)
    lc.set_array(np.linspace(0.,1.,len(segments)))
    return lc




def OrbitPlotOneSlice(sim, ax, lim=None, limz=None, Narc=100, color=False, periastron=False, trails=False, show_orbit=True, lw=1., axes="xy", plotparticles=[], primary=None, glow=False, fancy=False):
    p_orb_pairs = []
    if not plotparticles:
        plotparticles = range(1, sim.N_real)
    for i in plotparticles:
        p = sim.particles[i]
        p_orb_pairs.append((p, p.calculate_orbit(primary=primary)))

    if lim is None:
        lim = 0.
        for p, o in p_orb_pairs: 
            if o.a>0.:
                r = (1.+o.e)*o.a
            else:
                r = o.d
            if r>lim:
                lim = r
        lim *= 1.15
    if limz is None:
        z = [p.z for p,o in p_orb_pairs]
        limz = 2.0*max(z)
        if limz > lim:
            limz = lim
        if limz <= 0.:
            limz = lim

    if axes[0]=="z":
        ax.set_xlim([-limz,limz])
    else:
        ax.set_xlim([-lim,lim])
    if axes[1]=="z":
        ax.set_ylim([-limz,limz])
    else:
        ax.set_ylim([-lim,lim])
        
    if fancy:
        ax.set_facecolor((0.,0.,0.))
        for pos in ['top', 'bottom', 'right', 'left']:
            ax.spines[pos].set_edgecolor((0.3,0.3,0.3))

    if color is not False:
        if isinstance(color, list):
            colors = []
            for c in color:
                colors.append(get_color(c))
        elif isinstance(color, str):
            colors = [get_color(color)]
        elif color == True:
            colors = [(1.,0.,0.),(0.,0.75,0.75),(0.75,0.,0.75),(0.75, 0.75, 0,),(0., 0., 0.),(0., 0., 1.),(0., 0.5, 0.)]
    else:
        if fancy:
            colors = [(181./206.,66./206.,191./206.)]
            glow = True
        else:
            colors = ["black"]
    coloriterator = cycle(colors)

#    coords = {'x':0, 'y':1, 'z':2}
#    axis0 = coords[axes[0]]
#    axis1 = coords[axes[1]]
   
    prim = sim.particles[0] if primary is None else primary 
    if fancy:
        sun = (256./256.,256./256.,190./256.)
        opa = 0.020
        size = 6000.
        for i in range(256):
            ax.scatter(getattr(prim,axes[0]),getattr(prim,axes[1]), alpha=opa, s=size*lw, facecolor=sun, edgecolor=None, zorder=3)
            size *= 0.95
        
        starcolor = (1.,1.,1.)
        mi, ma = ax.get_xlim()
        prestate = random.getstate()
        random.seed(1) #always same stars
        x, y = [], []
        #small stars
        for i in range(64):
            x.append(random.uniform(mi,ma))
            y.append(random.uniform(mi,ma))
        ax.scatter(x,y, alpha=0.05, s=8*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        ax.scatter(x,y, alpha=0.1, s=4*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        ax.scatter(x,y, alpha=0.2, s=0.5*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        #medium stars
        x, y = [], []
        for i in range(16):
            x.append(random.uniform(mi,ma))
            y.append(random.uniform(mi,ma))
        ax.scatter(x,y, alpha=0.1, s=15*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        ax.scatter(x,y, alpha=0.1, s=5*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        ax.scatter(x,y, alpha=0.5, s=2*lw, facecolor=starcolor, edgecolor=None, zorder=3)
        random.setstate(prestate)

    else:
        ax.scatter(getattr(prim,axes[0]),getattr(prim,axes[1]), marker="*", s=35*lw, facecolor="black", edgecolor=None, zorder=3)
    
    proj = {}
    for p, o in p_orb_pairs:
        colori = next(coloriterator)
        
        prim = p.jacobi_com if primary is None else primary 
        if fancy:
            ax.scatter(getattr(p,axes[0]), getattr(p,axes[1]), s=25*lw, facecolor=colors, edgecolor=None, zorder=3)
        else:
            pass
            #::: !!!
            #::: !!!
            #::: plot the planet symbols
            #::: !!!
            #::: !!!
#            ax.scatter(getattr(p,axes[0]), getattr(p,axes[1]), s=p.r*lw, facecolor=colori, edgecolor=None, zorder=3)

       
        if show_orbit is True:
            alpha_final = 0. if trails is True else 1. # fade to 0 with trails

            hyperbolic = o.a < 0. # Boolean for whether orbit is hyperbolic
            if hyperbolic is False:
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim))
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_final=alpha_final, lw=lw, glow=glow)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)

            else:
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim, useTrueAnomaly=False))
                # true anomaly stays close to limiting value and switches quickly at pericenter for hyperbolic orbit, so use mean anomaly
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_final=alpha_final, lw=lw, glow=glow)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)
          
                alpha = 0.2 if trails is True else 1.
                pts = np.array(p.sample_orbit(Npts=Narc+1, primary=prim, trailing=False, useTrueAnomaly=False))
                proj['x'],proj['y'],proj['z'] = [pts[:,i] for i in range(3)]
                lc = fading_line(proj[axes[0]], proj[axes[1]], colori, alpha_initial=alpha, alpha_final=alpha, lw=lw, glow=glow)
                if type(lc) is list:
                    for l in lc:
                        ax.add_collection(l)
                else:
                    ax.add_collection(lc)

        if periastron:
            newp = Particle(a=o.a, f=0., inc=o.inc, omega=o.omega, Omega=o.Omega, e=o.e, m=p.m, primary=prim, simulation=sim)
            ax.plot([getattr(prim,axes[0]), getattr(newp,axes[0])], [getattr(prim,axes[1]), getattr(newp,axes[1])], linestyle="dotted", c=colori, zorder=1, lw=lw)
            ax.scatter([getattr(newp,axes[0])],[getattr(newp,axes[1])], marker="o", s=5.*lw, facecolor="none", edgecolor=colori, zorder=1)



    


def plot_top_down_view(params_median, params_star, timestep=None, scaling=30., plot_arrow=False):
    
    sim = rebound.Simulation()
    sim.add(m=1)
    
    for i, companion in enumerate(config.BASEMENT.settings['companions_all']):
        if timestep is None:
            timestep = params_median[companion+'_epoch']
        first_epoch = get_first_epoch(timestep, params_median[companion+'_epoch'], params_median[companion+'_epoch'])
        phase = calc_phase(timestep, params_median[companion+'_period'], first_epoch)
        ecc = params_median[companion+'_f_s']**2 + params_median[companion+'_f_c']**2
        w = np.arccos( params_median[companion+'_f_c'] / np.sqrt(ecc) ) #in rad
        inc = params_median[companion+'_incl']/180.*np.pi
        a = params_star['R_star'] / params_median[companion+'_radius_1'] #in Rsun 
        a *= 0.004650467260962157 #in AU
#        print(a, inc, ecc, w, phase*2*np.pi)
        if ecc>0:
            sim.add(a=a, inc=inc-np.pi/2., e=ecc, omega=w, f=phase*2*np.pi)
        else:
            sim.add(a=a, inc=inc--np.pi/2., f=phase*2*np.pi)
    
    fig = OrbitPlot(sim, xlabel='AU', ylabel='AU', color=[sns.color_palette('deep')[i] for i in [0,1,3]], lw=2)
    ax = plt.gca()
    
    for i, companion in enumerate(config.BASEMENT.settings['companions_all']):
        
        R_companion = params_star['R_star'] * params_median[companion+'_rr'] # in Rsun
        R_companion *= 0.004650467260962157 #in AU
        R_companion *= scaling
    
        
        x = sim.particles.get(i+1).x
        y = sim.particles.get(i+1).y
        p = Circle((x,y), R_companion, color=np.array(sns.color_palette('deep'))[[0,1,3],:][i])
        ax.add_artist(p)
        
    if plot_arrow:
        x0, x1 = ax.get_xlim()
        plt.arrow( 0.1*x1, 0, 0.7*x1, 0, color='silver', zorder=1 ) 
              
    plt.axis('equal')
    ax.set(xlim=[-0.12,0.12], ylim=[-0.12,0.12])
    loc = plticker.MultipleLocator(base=0.05) # this locator puts ticks at regular intervals
    ax.xaxis.set_major_locator(loc)
    ax.yaxis.set_major_locator(loc)

    return fig, ax


    
    



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
"""
Created on Tue Nov 27 15:41:12 2018

@author:
Maximilian N. Günther
MIT Kavli Institute for Astrophysics and Space Research, 
Massachusetts Institute of Technology,
77 Massachusetts Avenue,
Cambridge, MA 02109, 
USA
Email: maxgue@mit.edu
Web: www.mnguenther.com
"""

from __future__ import print_function, division, absolute_import

#::: plotting settings
import seaborn as sns
sns.set(context='paper', style='ticks', palette='deep', font='sans-serif', font_scale=1.5, color_codes=True)
sns.set_style({"xtick.direction": "in","ytick.direction": "in"})
sns.set_context(rc={'lines.markeredgewidth': 1})

#::: modules
import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import os
from tqdm import tqdm
import matplotlib.gridspec as gridspec
from mpl_toolkits.axes_grid1 import make_axes_locatable
from itertools import product

#::: my modules
import allesfitter
from allesfitter import config





###############################################################################
#::: plot publication ready spot plot
#::: flux
#::: model for 20 samples
#::: residuals for 1 sample
#::: 2D-spot map for 1 sample
###############################################################################

def plot_publication_spots_from_posteriors(datadir, Nsamples=20, command='save', mode='default'):
    '''
    command : str
        'show', 'save', 'return', 'show and return', 'save and return'
    mode: str
        default : 5000 points, phase (-0.25,0.75), errorbars
        zhan2019 : 100 points, phase (0,2), no errorbars
    '''
    
    fig, ax1, ax2, ax3 = setup_grid()
        
    config.init(datadir)
    posterior_samples = allesfitter.get_ns_posterior_samples(datadir, Nsamples=Nsamples, as_type='2d_array')
    
    for inst in config.BASEMENT.settings['inst_all']:
        if config.BASEMENT.settings['host_N_spots_'+inst] > 0:
            
            if mode=='default':
                xx = np.linspace(config.BASEMENT.data[inst]['time'][0], config.BASEMENT.data[inst]['time'][-1], 5000)
            elif mode=='zhan2019':
                xx = np.linspace(0, 2, 10000)

            for i_sample, sample in tqdm(enumerate(posterior_samples)):
            
                params = allesfitter.computer.update_params(sample)
                    
                spots = [ [params['host_spot_'+str(i)+'_long_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                          [params['host_spot_'+str(i)+'_lat_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                          [params['host_spot_'+str(i)+'_size_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                          [params['host_spot_'+str(i)+'_brightness_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ] ]  
    
    
                model = allesfitter.computer.calculate_model(params, inst, 'flux')
                baseline = allesfitter.computer.calculate_baseline(params, inst, 'flux')

                model_xx = allesfitter.computer.calculate_model(params, inst, 'flux', xx=xx) #evaluated on xx (!)
                baseline_xx = allesfitter.computer.calculate_baseline(params, inst, 'flux', xx=xx) #evaluated on xx (!)
    
                if i_sample==0:
                    if mode=='default':
                        ax1 = axplot_data(ax1, config.BASEMENT.data[inst]['time'], config.BASEMENT.data[inst]['flux'], flux_err=np.exp(params['log_err_flux_'+inst]))
                        ax2 = axplot_residuals(ax2, config.BASEMENT.data[inst]['time'], config.BASEMENT.data[inst]['flux']-model-baseline, res_err=np.exp(params['log_err_flux_'+inst]))
                        ax3 = axplot_spots_2d(ax3, spots)  
                    elif mode=='zhan2019':
                        ax1 = axplot_data(ax1, 
                                          np.concatenate(( config.BASEMENT.data[inst]['time'], config.BASEMENT.data[inst]['time']+1, config.BASEMENT.data[inst]['time']+2 )), 
                                          np.concatenate(( config.BASEMENT.data[inst]['flux'], config.BASEMENT.data[inst]['flux'], config.BASEMENT.data[inst]['flux'] )), 
                                          flux_err=None)
                        ax2 = axplot_residuals(ax2, 
                                               np.concatenate(( config.BASEMENT.data[inst]['time'], config.BASEMENT.data[inst]['time']+1, config.BASEMENT.data[inst]['time']+2 )), 
                                               np.concatenate(( config.BASEMENT.data[inst]['flux']-model-baseline, config.BASEMENT.data[inst]['flux']-model-baseline, config.BASEMENT.data[inst]['flux']-model-baseline )), 
                                               res_err=None)
                        ax3 = axplot_spots_2d(ax3, spots)      
                    
                ax1 = axplot_model(ax1, xx, model_xx+baseline_xx)  
                
            ax1.locator_params(axis='y', nbins=5)
            
            if mode=='zhan2019':
                ax1.set(xlim=[0,2])
                ax2.set(xlim=[0,2])
        
            if 'save' in command:
                pubdir = os.path.join(config.BASEMENT.outdir, 'pub')
                if not os.path.exists(pubdir): os.makedirs(pubdir)
                if mode=='default':
                    fig.savefig( os.path.join(pubdir,'host_spots_'+inst+'.pdf'), bbox_inches='tight' )
                elif mode=='zhan2019':
                    fig.savefig( os.path.join(pubdir,'host_spots_'+inst+'_zz.pdf'), bbox_inches='tight' )
                plt.close(fig)
    
            if 'show' in command:
                plt.show()
                
            if 'return' in command:
                return fig, ax1, ax2, ax3
                    
                    

def setup_grid():
    fig = plt.figure(figsize=(8,3.8))
    
    gs0 = gridspec.GridSpec(1, 2)
    
    gs00 = gridspec.GridSpecFromSubplotSpec(4, 1, subplot_spec=gs0[0], hspace=0)
    ax1 = plt.Subplot(fig, gs00[:-1, :])
    ax1.set(xlabel='', xticks=[], ylabel='Flux')
    fig.add_subplot(ax1)
    ax2 = plt.Subplot(fig, gs00[-1, :])
    ax2.set(xlabel='Phase', ylabel='Res.')
    fig.add_subplot(ax2)
    
    
    gs01 = gridspec.GridSpecFromSubplotSpec(1, 1, subplot_spec=gs0[1])
    ax3 = plt.Subplot(fig, gs01[:, :])
    ax3.set(xlabel='Long. (deg)', ylabel='Lat. (deg.)')
    fig.add_subplot(ax3)
    
    plt.tight_layout()
    return fig, ax1, ax2, ax3
        
        

def axplot_data(ax, time, flux, flux_err=None):
    if flux_err is not None:
        ax.errorbar(time, flux,  yerr=flux_err, marker='.', linestyle='none', color='lightblue', zorder=9)
    ax.plot(time, flux, 'b.', zorder=10)
    return ax


def axplot_residuals(ax, time, res, res_err=None):
    if res_err is not None:
        ax.errorbar(time, res,  yerr=res_err, marker='.', linestyle='none', color='lightblue', zorder=9)
    ax.plot(time, res, 'b.', zorder=10)
    ax.axhline(0, color='r', linewidth=2, zorder=11)
    return ax
    

def axplot_model(ax, time, model):
    ax.plot(time, model, 'r-', zorder=11)
    return ax



        
def axplot_spots_2d(ax, spots):
    '''
    Inputs:
    -------
    spots : ...
    
    command : str
        'show' : show the figure (do not automatically save)
        'return' : return the figure object (do not display)
    
    e.g. for two spots:
    spots = [ [lon1, lon2],
              [lat1, lat2],
              [size1, size2],
              [brightness1, brightness2] ] 
    '''

    np.random.seed(42)

    spots = np.array(spots)
    N_spots = len(spots[0])
    
    for i in range(N_spots):
        lon, lat, size, brightness = spots[:,i]
        
        r = size
        theta = np.linspace(0, 2*np.pi, 100)
        lonv = lon + r * np.cos(theta)
        latv = lat + r * np.sin(theta)

        cm = plt.get_cmap('coolwarm')
        color = cm(brightness/2.)
        sc = ax.scatter(lon, lat, c=brightness, cmap='coolwarm', vmin=0, vmax=2)
        
        a = [-360.,0.,360.]
        b = [-180.,0.,180.]
        for r in product(a, b): 
            ax.fill(lonv+r[0], latv+r[1], color=color)
            ax.plot(lonv+r[0], latv+r[1], 'k-')
        
    divider = make_axes_locatable(ax)
    cax = divider.append_axes('right', size='5%', pad=0.05)
    cbar = plt.colorbar(sc, cax=cax, ticks=[0,0.5,1,1.5,2])
    cbar.set_label('Brightness')
    ax.set(xlim=[0,360], ylim=[-90,90], xticks=[0,90,180,270,360], yticks=[-90,-45,0,45,90])
    
    return ax

                        
  

###############################################################################
#::: plot the spotmaps plots
#::: 3D-spot-map and 2D-spot-map, individually for 10 samples
###############################################################################                      

def plot_spots_from_posteriors(datadir, Nsamples=10, command='return'):
    
    if command=='show':
        Nsamples = 1 #overwrite user input and only show 1 sample if command=='show'
        
        
    config.init(datadir)
    posterior_samples_dic = allesfitter.get_ns_posterior_samples(datadir, Nsamples=Nsamples)
    
    for sample in tqdm(range(Nsamples)):
    
        params = {}
        for key in posterior_samples_dic:
            params[key] = posterior_samples_dic[key][sample]
        
        
        for inst in config.BASEMENT.settings['inst_all']:
            
            if config.BASEMENT.settings['host_N_spots_'+inst] > 0:
                spots = [
                                     [params['host_spot_'+str(i)+'_long_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                                     [params['host_spot_'+str(i)+'_lat_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                                     [params['host_spot_'+str(i)+'_size_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ],
                                     [params['host_spot_'+str(i)+'_brightness_'+inst] for i in range(1,config.BASEMENT.settings['host_N_spots_'+inst]+1) ]
                                    ]
        
                if command=='return':
                    fig, ax, ax2 = plot_spots(spots, command='return')
                    plt.suptitle('sample '+str(sample))
                    spotsdir = os.path.join(config.BASEMENT.outdir, 'spotmaps')
                    if not os.path.exists(spotsdir): os.makedirs(spotsdir)
                    fig.savefig( os.path.join(spotsdir,'host_spots_'+inst+'_posterior_sample_'+str(sample)) )
                    plt.close(fig)
                
                elif command=='show':
                    plot_spots(spots, command='show')
        
        for companion in config.BASEMENT.settings['companions_all']:
            for inst in config.BASEMENT.settings['inst_all']:
                if config.BASEMENT.settings[companion+'_N_spots_'+inst] > 0:
                    spots = [
                                         [params[companion+'_spot_'+str(i)+'_long_'+inst] for i in range(1,config.BASEMENT.settings[companion+'_N_spots_'+inst]+1) ],
                                         [params[companion+'_spot_'+str(i)+'_lat_'+inst] for i in range(1,config.BASEMENT.settings[companion+'_N_spots_'+inst]+1) ],
                                         [params[companion+'_spot_'+str(i)+'_size_'+inst] for i in range(1,config.BASEMENT.settings[companion+'_N_spots_'+inst]+1) ],
                                         [params[companion+'_spot_'+str(i)+'_brightness_'+inst] for i in range(1,config.BASEMENT.settings[companion+'_N_spots_'+inst]+1) ]
                                        ]
                    
                    if command=='return':
                        fig, ax, ax2 = plot_spots(spots, command='return')
                        plt.suptitle('sample '+str(sample))
                        spotsdir = os.path.join(config.BASEMENT.outdir, 'spotmaps')
                        if not os.path.exists(spotsdir): os.makedirs(spotsdir)
                        fig.savefig( os.path.join(spotsdir,companion+'_spots_'+inst+'_posterior_sample_'+str(sample)) )
                        plt.close(fig)
                    
                    elif command=='show':
                        plot_spots(spots, command='show')
                
                


def plot_spots(spots, command='show'):
    '''
    Inputs:
    -------
    spots : ...
    
    command : str
        'show' : show the figure (do not automatically save)
        'return' : return the figure object (do not display)
    
    e.g. for two spots:
    spots = [ [lon1, lon2],
              [lat1, lat2],
              [size1, size2],
              [brightness1, brightness2] ] 
    '''

    np.random.seed(42)

    spots = np.array(spots)
    N_spots = len(spots[0])
    radius = 1.
    N_rand = 3000
    
    fig = plt.figure(figsize=(10,4))
    ax = fig.add_subplot(121, projection='3d')
    ax2 = fig.add_subplot(122)
    
    u = np.linspace(0, 2 * np.pi, 100)
    v = np.linspace(0, np.pi, 100)
    x = radius * np.outer(np.cos(u), np.sin(v))
    y = radius * np.outer(np.sin(u), np.sin(v))
    z = radius * np.outer(np.ones(np.size(u)), np.cos(v))
    ax.plot_surface(x, y, z, color='orange', linewidth=0, rstride=4, cstride=4, alpha=0.2, antialiased=False, shade=False)
    
    
    lon = np.linspace( 0./180.*np.pi, 360./180.*np.pi, 100 )
    lat = np.zeros_like(lon)
    x = radius * np.cos(lon) * np.cos(lat)
    y = radius * np.sin(lon) * np.cos(lat)
    z = radius * np.sin(lat)
    ax.plot(x,y,z,ls='-',c='grey',alpha=0.3)
    
    
    lat = np.linspace( 0./180.*np.pi, 360./180.*np.pi, 100 )
    lon = np.zeros_like(lat)
    x = radius * np.cos(lon) * np.cos(lat)
    y = radius * np.sin(lon) * np.cos(lat)
    z = radius * np.sin(lat)
    ax.plot(x,y,z,ls='-',c='grey',alpha=0.3)
    
    
    for i in range(N_spots):
        lon, lat, size, brightness = spots[:,i]
        
        r = size * np.sqrt(np.random.rand(N_rand))
        theta = np.random.rand(N_rand) * 2. * np.pi
        lonv = lon + r * np.cos(theta)
        latv = lat + r * np.sin(theta)
        
        lon = lonv/180.*np.pi
        lat = latv/180.*np.pi
        x = radius * np.cos(lon) * np.cos(lat)
        y = radius * np.sin(lon) * np.cos(lat)
        z = radius * np.sin(lat)
        c = brightness * np.ones_like(lon)
        sc = ax.scatter(x,y,z,c=c,marker='.', cmap='seismic', vmin=0, vmax=2, alpha=1, rasterized=True)
        ax2.scatter(lonv, latv, c=c, marker='.', cmap='seismic', vmin=0, vmax=2, alpha=1, rasterized=True)
        
        
        lon, lat, size, brightness = spots[:,i]
        
        r = size
        theta = np.linspace(0, 2*np.pi, 100)
        lonv = lon + r * np.cos(theta)
        latv = lat + r * np.sin(theta)
        
        lon = lonv/180.*np.pi
        lat = latv/180.*np.pi
        x = radius * np.cos(lon) * np.cos(lat)
        y = radius * np.sin(lon) * np.cos(lat)
        z = radius * np.sin(lat)
        ax.plot(x,y,z,'k-',zorder=20)
        ax2.plot(lonv, latv,'k-',zorder=20)
        
    plt.colorbar(sc)
        
    ax2.set(xlim=[0,360], ylim=[-90,90])
    
        
    
    if command=='return':
        return fig, ax, ax2

    elif command=='show':
        plt.show()
    



"""
Predict JWST transits for obtained posteriors

argument 1: name of the planet

Tansu Daylan
MIT Kavli Institute, Cambridge, MA, 02109, US
tansu.daylan@gmail.com
www.tansudaylan.com
"""

import numpy as np
import os, sys
import pickle
from dynesty import utils as dyutils
from allesfitter import config
from astropy.time import Time
import matplotlib.pyplot as plt


# base data path for the TESS known planet project
pathdata = os.environ['KNWN_DATA_PATH'] + '/'
pathdatapost = os.environ['KNWN_DATA_PATH'] + '/postproc/'

# name of the exoplanet
strgplan = sys.argv[1]


pathdataplan = pathdata + '%s/allesfit_global/allesfit_onlytess_full/' % strgplan
config.init(pathdataplan)
fileobjt = open(pathdataplan + 'results/save_ns.pickle', 'rb')
objtrest = pickle.load(fileobjt)
weig = np.exp(objtrest['logwt'] - objtrest['logz'][-1])
chan = dyutils.resample_equal(objtrest.samples, weig)
    
listkeys = config.BASEMENT.fitkeys
listlabl = config.BASEMENT.fitlabels

# get period and epoch posteriors
for k, labl in enumerate(listlabl):
    if listkeys[k] == 'b_epoch':
        postepoc = chan[:, k]
    if listkeys[k] == 'b_period':
        postperi = chan[:, k]

# the time at which the transit is to be predicted
timejwst = '2022-01-01T00:00:00'


objttime = Time(timejwst, format='isot', scale='utc')

indxtranjwst = 1000#np.argmax(objttime.jd - 

# posterior of the predicted transit
postepocjwst = postepoc + indxtranjwst * postperi

# plot the posterior
figr, axis = plt.subplots()
axis.hist(postepocjwst)
path = pathdatapost + 'posttranjwst_%s.png' % strgplan
print 'Writing to %s...' % path
plt.savefig(path)
plt.close()






#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Oct  5 14:18:20 2018

@author:
Maximilian N. Günther
MIT Kavli Institute for Astrophysics and Space Research, 
Massachusetts Institute of Technology,
77 Massachusetts Avenue,
Cambridge, MA 02109, 
USA
Email: maxgue@mit.edu
Web: www.mnguenther.com
"""

from __future__ import print_function, division, absolute_import

#::: plotting settings
import seaborn as sns
sns.set(context='paper', style='ticks', palette='deep', font='sans-serif', font_scale=1.5, color_codes=True)
sns.set_style({"xtick.direction": "in","ytick.direction": "in"})
sns.set_context(rc={'lines.markeredgewidth': 1})

#::: modules
import os
import gzip
try:
   import cPickle as pickle
except:
   import pickle




#:::: allesfitter modules
from . import config

from .mcmc import mcmc_fit
from .nested_sampling import ns_fit

from . import general_output
from . import nested_sampling_output

from .general_output import show_initial_guess, get_labels, get_data, get_settings
from .nested_sampling_output import get_ns_posterior_samples, get_ns_params, ns_output
from .mcmc_output import get_mcmc_posterior_samples, mcmc_output

from .computer import calculate_model

from .priors import transform_priors
from .priors.estimate_noise import estimate_noise

from .postprocessing.nested_sampling_compare_logZ import ns_plot_bayes_factors
from .postprocessing.plot_violins import ns_plot_violins, mcmc_plot_violins

def GUI():
    allesfitter_path = os.path.dirname( os.path.realpath(__file__) )
    os.system( 'jupyter notebook "' + os.path.join(allesfitter_path,'GUI.ipynb') + '"')


class allesclass():
    def __init__(self,datadir):
        config.init(datadir)
        self.data = config.BASEMENT.data
        self.settings = config.BASEMENT.settings
        
        try:
            f = gzip.GzipFile(os.path.join(config.BASEMENT.outdir,'save_ns.pickle.gz'), 'rb')
            results = pickle.load(f)
            f.close()
            self.posterior_samples = nested_sampling_output.draw_ns_posterior_samples(results)                               # all weighted posterior_samples
            self.posterior_params_median, self.posterior_params_ll, self.posterior_params_ul = general_output.get_params_from_samples(self.posterior_samples)
        except:
            pass
        
    def get_median_model(self, inst, key, xx=None):
        return calculate_model(self.posterior_params_median, inst, key, xx=xx)
        

#::: version
__version__ = '0.8.2'